#include <vector>
#include <string>
#include <iostream>

#include "generated.h"


using std::vector;
using std::string;
using std::cout;
using std::endl;


// Wrapper for the class generated by the compiler
class WaterSensor {
  cell_lang::WaterSensor target;

public:
  class Observer {
  public:
    virtual void on_state_change(const string &new_state) = 0;
  };


  WaterSensor() {
    target.apply();
  }

  void failed_reading() {
    target.set_input(cell_lang::WaterSensor::RAW_READING, "nothing");
    // The following line is equivalent to the above one:
    // target.set_raw_reading(NULL);
  }

  void new_reading(bool value) {
    target.set_input(cell_lang::WaterSensor::RAW_READING, value ? "just(true)" : "just(false)");
    // The following line is equivalent to the above one:
    // target_set_raw_reading(value);
  }

  void apply(unsigned int elapsed_secs, Observer &observer) {
    bool done;
    target.set_elapsed_secs(elapsed_secs);
    do {
      done = target.apply();
      for (cell_lang::WaterSensor::Output id : target.changed_outputs()) {
        string value = target.read_output(id);
        switch (id) {
          case cell_lang::WaterSensor::SENSOR_STATE:
            observer.on_state_change(value);
            break;
        }
      }
    } while (!done);
  }
};


class Writer : public WaterSensor::Observer {
public:
  void on_state_change(const string &new_value) {
    cout << new_value << endl;
  }
};


int main(int argc, char **argv) {
  Writer writer;
  WaterSensor sensor;

  sensor.new_reading(false);
  sensor.apply(40, writer);

  sensor.failed_reading();
  sensor.apply(10, writer);

  sensor.failed_reading();
  sensor.apply(10, writer);

  sensor.failed_reading();
  sensor.apply(10, writer);

  sensor.new_reading(true);
  sensor.apply(10, writer);

  sensor.new_reading(false);
  sensor.apply(50, writer);
}
